<template>
  <div id="create-signs-shop">
    <h1 v-show="accountName">Create Sign for {{ accountName }}</h1>
    <form
      id="create-signs-form">
      <header>
        <h1>Sign Options</h1>
      </header>
      <section style="display: inline-block; width:47%; margin-right:4%;">
        <h2>Images and Text</h2>
        <a
          @click="$refs.clipartModal.Open()"
          class="button popup">Add Image
        </a>
        <a
          @click="addText"
          class="button popup">Add Text
        </a>
      </section>
      <section style="display: inline-block; width: 47%;">
        <h2>Borders</h2>
        <a
          @click="$refs.borderModal.Open()"
          class="button popup">Add Border
        </a>
        <a
          v-show="borderExists"
          @click="removeBorder"
          class="button popup">Remove Border
        </a>
      </section>
      <section
        v-show="imgSelected"
        class="canvasRibbon">
        <label v-if="imgSelected" class="fontSelectLabel">Image</label>
        <a
          @click="deleteObj"
          class="button popup">Delete Image
        </a>
        <a
          @click="sendToBack"
          class="button popup">Send To Back
        </a>
      </section>
      <section
        v-show="textSelected"
        class="canvasRibbon">
        <div
          v-show="textSelected"
          class="fontContainer">
          <label v-if="textSelected" class="fontSelectLabel">Font</label>
          <select
            id="fontSelect"
            name="fontSelect"
            @change="updateFont"
            v-model=" selectedFont">
            <option
              value="Mouse Memoirs"
              selected>Mouse Memoirs
            </option>
            <option value="pointy_demoregular">Pointy</option>
            <option value="chunkfiveroman">Chunk Five</option>
            <option value="open_sansregular">Open Sans</option>
            <option value="pacificoregular">Pacifico</option>
            <option value="Bungee Shade">Bungee Shade</option>
          </select>
          <a
            @click="deleteObj"
            v-show="textSelected"
            class="button popup">Delete Text
          </a>
          <a
            @click="sendToBack"
            class="button popup">Send To Back
          </a>
        </div>
      </section>
    </form>
    <!-- These div classes might be able to port to something else, legacy I think. -->
    <div class="signCanvas editor">
      <canvas
        id='canvas'
        ref='canvas'>
      </canvas>
    </div>
      <a
        @click.prevent="canvasSave"
        class="button save">Save Sign
      </a>
      <a
        @click.prevent="SavePrint"
        class="button print">Save & Print Sign
      </a>
    <shop-modal ref="clipartModal">
      <div slot="clipart" id="overlay">
        <div
          id="select-clipart-popup"
          class="signAndPrint modalwindow"
          style="top: 6em; left: 8em;">
          <div class="">
            <h1>Select Images</h1>
            <div class="ajaxContainer">
              <h1 v-if="!selectedCategory">Select an image category below to get started!</h1>
              <template v-for="image in selectedCategory">
                <div
                  @click="toggleSelectedStyle(image, selectedCategory)"
                  :key="image.id"
                  :class="{selected: image.selected}"
                  :style="{'background-image': `url(${image.stub})`}">
                  <img
                    class="image"
                    :src="image.stub" >
                </div>
              </template>
            </div>
            <div class="gradient"></div>
            <div class="popup-controls">
              <select
                v-model="selectedCategory"
                name="categories"
                class="categories">
                <option disabled value="">Choose Category</option>
                <option
                  v-for="(set, index) in images"
                  :key="index"
                  :value="set.src">{{ set.category }}</option>
              </select>
              <a
                @click="addImage"
                v-show="imageSelected"
                class="button submit choose">Choose Selected Image
              </a>
            </div>
          </div>
        <a @click="$refs.clipartModal.Close(); resetModals()" class="close closebutton maskfadeout" data-dismiss="modal">
          <img src="/images/closebutton.png">
        </a>
        </div>
      </div>
    </shop-modal>
    <shop-modal ref="borderModal">
      <div slot="border" id="overlay">
        <div
          id="select-border-popup"
          class="signAndPrint modalwindow"
          style="top: 6em; left: 8em;">
          <div class="windowcontent">
            <h1>Select Border</h1>
            <div class="ajaxContainer">
              <template v-for="border in borders">
                <div
                  @click="toggleSelectedStyle(border, borders)"
                  :key="border.id"
                  :class="{selected: border.selected}"
                  :style="{'background-image': `url(${border.stub})`}">
                  <img
                    class="image"
                    :src="border.stub" >
                </div>
              </template>
            </div>
          </div>
          <div class="gradient"></div>
          <div class="popup-controls">
            <a
              @click="addBorder"
              v-show="imageSelected"
              class="button submit choose">Choose Selected Border
            </a>
          </div>
        </div>
        <a @click="$refs.borderModal.Close(); resetModals()" class="close closebutton maskfadeout" data-dismiss="modal">
          <img src="/images/closebutton.png">
        </a>
      </div>
    </shop-modal>
  </div>
</template>

<script>
import ShopModal from '../../../../shared/views/ShopModal.vue';

export default {
  components: {
    ShopModal
  },
  mounted: function () {
    this.instantiateFabric();
  },
  created:function(){
    this.getImageRefs();
  },
  data() {
    return {
      images: [],
      borders:[],
      selectedCategory: '',
      selectedImageSrc: '',
      imageSelected: false,
      selectedFont: 'Mouse Memoirs',
      textSelected: false,
      imgSelected: false,
      objSelected: false,
      canvasObject: Object,
      accountId: this.$route.params.accountId,
      accountName: '',
      borderExists: false,
    }
  },
  props: {
    shop: Object
  },
  methods: {
    instantiateFabric() {
      //set snap angle
      fabric.Object.prototype.set({
          snapThreshold: 5,
          snapAngle: 45
      });
      this.$refs.canvas = new fabric.Canvas('canvas', {
        backgroundColor: "#ffffff",
        //letter size at 72dpi in pixels
        width: 612,
        height: 792,
        controlsAboveOverlay : true,
        //preserveObjectStacking : false,
      });
      this.canvasInit();
      this.getAccountName();
    },
    getAccountName(){
      app.api.Get("sign_and_print/getsignnamebyaccountid/" + this.accountId,)
        .then( response => {
          this.accountName = response;
        })
        .catch( x => app.Message( x.error, "error" ));
    },
    // load canvas object from db or create initial object
    canvasInit(){
      app.api.Get("sign_and_print/getsignbyaccountid/" + this.accountId,)
        .then( response => {
          this.canvasObject = response.canvas;
          this.$refs.canvas.loadFromJSON(this.canvasObject);
          this.$refs.canvas.renderAll();
          // this is a toplevel callback for anytime an object is modified
          var self = this;
          this.$refs.canvas.on('object:modified', self.canvasUpdate);
          this.$refs.canvas.on('object:selected', self.onObjectSelected);
          this.$refs.canvas.on('selection:updated', self.onObjectSelected);
          this.$refs.canvas.on('selection:cleared', self.onSelectionClear);
        })
        .catch( x => app.Message( x.error, "error" ));
    },
    // push updates of model to backend
    canvasSave(){
      app.api.Post("sign_and_print/savesign/", {
        canvas: this.canvasObject,
        accountId: this.accountId
      })
        .then( Response => {
          console.log(Response);
        })
        .catch( x => app.Message( x.error, "error" ));
    },
    // this is mainly for the "object:modified" loop, but call this everytime state changes on the canvas object
    canvasUpdate(){
      console.log('updated');

      this.canvasObject = this.$refs.canvas.toJSON();
      this.canvasSave();
    },
    // doCanvasThings() {
    //    this.$refs.canvas.on('object:selected', self.onObjectSelected);
    //   canvas.on('selection:cleared', function() {
    //   ...
    //   });
    // },
    onSelectionClear(e) {
      console.log(e);
    },
    onObjectSelected(e) {
      console.log('selected or updated');

      let objType = e.target.get('type');
      this.textSelected = false;
      this.imgSelected = false;
      // something is selected
      this.objSelected = true;
      if (objType == 'image') {
        this.imgSelected = true;
        this.textSelected = false;

      }
      else if (objType == 'textbox') {
        //TODO: have font dropdown show font of selected text box
        let selectedText = this.$refs.canvas.getActiveObject();
        this.textSelected = true;
        this.imgSelected = false;
        this.selectedFont = selectedText.get('fontFamily');
      }
      //control styles
      e.target.set({
        transparentCorners: false,
        borderColor: 'blue',
        cornerColor: 'white',
        cornerStrokeColor: 'blue'
      });
      this.$refs.canvas.bringToFront(e.target);
      this.canvasObject = this.$refs.canvas.toJSON();
    },
    updateFont() {
      let aObj = this.$refs.canvas.getActiveObject();
        if(aObj) {

          aObj.set("fontFamily", this.selectedFont);
          this.$refs.canvas.requestRenderAll();

        }
    },
    addText() {
      var tempText = new fabric.Textbox('Click to edit text');
        let l = 200;
        let t = 170;
        tempText.set({'left':l});
        tempText.set({'top':t});
        tempText.set({'textAlign': 'center'});
        tempText.setControlsVisibility({
                    mt: false,
                    mb: false,
                    // ml: false,
                    // mr: false,
                });
        this.selectedFont = "Mouse Memoirs"
        tempText.fontFamily = this.selectedFont;
      this.textSelected = true;
      this.$refs.canvas.add(tempText).setActiveObject(tempText).renderAll();
      this.canvasObject = this.$refs.canvas.toJSON();


    },
    addImage() {
      // use selectedImageSrc as the path to image to create Fabric object
      var self = this;
      fabric.Image.fromURL(this.selectedImageSrc, function(oImg) {
          let l = 150;
          let t = 200;
          oImg.scale(0.5);
          oImg.set({'left':l});
          oImg.set({'top':t});
          oImg.setControlsVisibility({
              mt: false,
              mb: false,
              ml: false,
              mr: false,
          });
          self.$refs.canvas.add(oImg).setActiveObject(oImg).renderAll();

      });
      this.selectedImageSrc = '';
      this.$refs.clipartModal.Close();
    },
    addBorder() {
      // use selectedImageSrc as the path to image to create Fabric object
      var canvas = this.$refs.canvas;
      var self = this;
      fabric.Image.fromURL(this.selectedImageSrc, function(oImg) {
        oImg.scaleToWidth(canvas.width);
        oImg.scaleToHeight(canvas.height);
        canvas.setOverlayImage(oImg, canvas.renderAll.bind(canvas));
        self.borderExists = true;
      });

      // fabric.Image.fromURL(this.selectedImageSrc, function(oImg) {

      //     oImg.set({'left':0});
      //     oImg.set({'top':0});
      //     oImg.evented = false;
      //     oImg.selectable = false;

      //     canvas.add(oImg).setActiveObject(oImg).renderAll();

      // });
      this.selectedImageSrc = '';
      this.$refs.borderModal.Close();
    },
    removeBorder() {
      var self = this;
      self.$refs.canvas.setOverlayImage(null, self.$refs.canvas.renderAll.bind(self.$refs.canvas));
      this.borderExists = false;
    },
    toggleSelectedStyle(element, array) {
      array.forEach(function (object) {
        if(object.id == element.id) {
          object.selected = true;
        } else {
          object.selected = false;
        }
      })
      this.imageSelected = true;
      this.selectedImageSrc = element.stub;
    },
    deleteObj() {
      let aObj = this.$refs.canvas.getActiveObject();
        if(aObj) {

          this.$refs.canvas.remove(aObj);
          this.objSelected = false;
          this.textSelected = false;
          this.imgSelected = false;
        }
    },
    sendToBack() {

      let aObj = this.$refs.canvas.getActiveObject();
        if(aObj) {

          this.$refs.canvas.sendToBack(aObj);
          this.$refs.canvas.discardActiveObject();

        }
    },
    SavePrint(){
      app.api.Post("sign_and_print/savesign/", {'canvas':this.canvasObject, 'accountId':this.accountId, 'completed':true})
        .then( Response => {
          // now that object is saved, send to printer
          var signMarkup = this.$refs.canvas.toSVG({suppressPreamble: true});
          app.printer.Sign(signMarkup);
          this.$router.push({name: 'sign_and_print_create_main'});
        })
        .catch( x => app.Message( x.error, "error" ));
    },
    resetModals() {
      this.selectedCategory = '';
      this.selectedImageSrc = '';
    },
    getImageRefs(){
      app.api.Get("images?type=signs").then( Response => {
        console.log(Response)
        this.borders = Response.borders;
        this.images = Response.images;
      })
      .catch( x => app.Message( x.error, "error" ));
    }
  },
  beforeRouteLeave (to, from, next) {
      app.api.Put("sign_and_print/checkInSign/" + this.accountId,)
      .then( Response => {
        next();
      })
      .catch( x => app.Message( x.error, "error" ));
  }
}
</script>

